/*
 * Copyright (C) 2008 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

//#define LOG_NDEBUG 0
#define LOG_TAG "NetlinkHandler"

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <log/log.h>
#include <sysutils/NetlinkEvent.h>
#include <hardware_legacy/uevent.h>
#include "NetlinkHandler.h"
#include "CameraMonitor.h"
#include "Debug.h"

namespace android_camera_hal3
{

NetlinkHandler::NetlinkHandler(int listenerSocket)
    : NetlinkListener(listenerSocket)
{
    uevent_init();
}

NetlinkHandler::~NetlinkHandler()
{
}

int NetlinkHandler::start()
{
    LOG_FUNCTION_NAME();
    return this->startListener();
}

int NetlinkHandler::stop()
{
    LOG_FUNCTION_NAME();
    return this->stopListener();
}

void NetlinkHandler::onEvent(NetlinkEvent* evt)
{
    ALOGV("%s[%d]: [CAMUPDATE] action:%d subsys:%s", __FUNCTION__, __LINE__, evt->getAction(), evt->getSubsystem());
    //evt->dump();

    bool bSwitch = false;
    bool bV4L2 = false;
    bool bPlatform = false;

    const char* subsys = evt->getSubsystem();

    if (subsys != NULL && strcmp(subsys, "switch") == 0) {
        ALOGD("[CAMUPDATE] switch update");
        bSwitch = true;
    }

    if (subsys != NULL && strcmp(subsys, "video4linux") == 0) {
        ALOGD("[CAMUPDATE] video4linux update");
        bV4L2 = true;
    }

#if 0
    if (subsys != NULL && strcmp(subsys, "platform") == 0) {
        ALOGD("[CAMUPDATE] platform update");
        bPlatform = true;
    }

    if (subsys != NULL && strcmp(subsys, "usb") == 0) {
        ALOGD("[CAMUPDATE] usb update");
        bV4L2 = true;
    }
#endif

    if(bSwitch || bV4L2) {
        CameraMonitor::Instance()->handleSwitchEvent(evt);
    }
}

};
